/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2009 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2IMGUISliderStyle.h 2393 2009-08-14 10:38:46Z tonic $
 * $Revision: 2393 $
 */

#ifndef T2IMGUISLIDERSTYLE_H_INCL
#define T2IMGUISLIDERSTYLE_H_INCL


namespace turska
{
    //! IMGUI slider style interface.
    class IMGUISliderStyle : public IMGUIWidgetStyle
    {
    public:
        virtual ~IMGUISliderStyle() {}

        //! Returns true if x,y is inside given rectangle.
        /*! If width or height is 0 then the style object should
         *  know the proper size by itself.
         */
        virtual bool isInside(const IMGUI &imgui, TREAL x, TREAL y,
                              const Rectangle &area) = 0;

        virtual void getIntModifierMultipliers(I32 &shift, I32 &control) = 0;
        virtual void getRealModifierMultipliers(TREAL &shift, TREAL &control) = 0;


        //! Returns integer value of point in slider.
        /*! \todo more detailed documentation
         */
        virtual I32 getIntValue(const IMGUI &imgui,
                                IMGUI_WIDGET_ORIENTATION axis,
                                I32 minValue, I32 maxValue,
                                TREAL x, TREAL y, const Rectangle &area) = 0;
        
        //! Returns integer value of point in slider.
        /*! \todo more detailed documentation
         */
        virtual TREAL getRealValue(const IMGUI &imgui,
                                  IMGUI_WIDGET_ORIENTATION axis,
                                  TREAL minValue, TREAL maxValue,
                                  TREAL x, TREAL y, const Rectangle &area) = 0;


        //! Renders a slider and returns covered area dimensions.
        /*! Implementation should check from IMGUI that rendering is enabled,
         *  or return without rendering anything if it is disabled.
         * \todo more detailed documentation
         */
        virtual Rectangle render(const IMGUI &imgui, I32 widgetState,
                                 IMGUI_WIDGET_ORIENTATION axis,
                                 I32 value, I32 minValue, I32 maxValue,
                                 TREAL x, TREAL y, TREAL width, TREAL height) = 0;
        
        //! Renders a slider and returns covered area dimensions.
        /*! Implementation should check from IMGUI that rendering is enabled,
         *  or return without rendering anything if it is disabled.
         * \todo more detailed documentation
         */
        virtual Rectangle render(const IMGUI &imgui, I32 widgetState,
                                 IMGUI_WIDGET_ORIENTATION axis,
                                 TREAL value, TREAL minValue, TREAL maxValue,
                                 TREAL x, TREAL y, TREAL width, TREAL height) = 0;
    };

} // namespace turska


#endif // !T2IMGUISLIDERSTYLE_H_INCL
