/* Copyright 2009 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: App.cpp 103 2009-12-11 18:56:44Z tonic $
 * $Revision: 103 $
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <time.h>
#include "main.h"


#define WINDOW_CAPTION_STR L"game name window caption --- change this in App.cpp"

#define BLURMIX(a,b,n) ((a) * ((n) - 1) + (b)) / (n)


App * App::mSingleton = NULL;


App::App(irr::IrrlichtDevice *irrDevice) :
    mAlive(true),
    mTime(0),
    mStartTime(0),
    mNextUpdateTime(0),
    mMaxLogicUpdatesPerFrame(4),
    mCurrentAppStateHandler(NULL),
    mMouseX(0), mMouseY(0),
    mIrrDevice(irrDevice),
    mIrrVideoDriver(NULL),
    mIrrSceneManager(NULL)
{
}


App::~App()
{
}


bool App::OnEvent(const irr::SEvent &event)
{
    if (mCurrentAppStateHandler == NULL)
        return false;

    switch (event.EventType)
    {
    case EET_GUI_EVENT:
        // TODO - forward irr gui events if necessary
        break;

    case EET_MOUSE_INPUT_EVENT:
        {
            /* for debug build, do not report events as handled so that
             * irr's built-in cameras get the events for debugging purposes.
             */
            /*
            #ifdef _DEBUG
            bool mouseEventHandledReturnValue = false;
            #else
            bool mouseEventHandledReturnValue = true;
            #endif
            */

            switch (event.MouseInput.Event)
            {
            case EMIE_LMOUSE_PRESSED_DOWN:
            case EMIE_MMOUSE_PRESSED_DOWN:
            case EMIE_RMOUSE_PRESSED_DOWN:
                return mCurrentAppStateHandler->onMouseDown(event.MouseInput);
                //return mouseEventHandledReturnValue;
            case EMIE_LMOUSE_LEFT_UP:
            case EMIE_MMOUSE_LEFT_UP:
            case EMIE_RMOUSE_LEFT_UP:
                return mCurrentAppStateHandler->onMouseUp(event.MouseInput);
                //return mouseEventHandledReturnValue;
            case EMIE_MOUSE_MOVED:
                mMouseX = event.MouseInput.X;
                mMouseY = event.MouseInput.Y;
                return mCurrentAppStateHandler->onMouseMotion(event.MouseInput);
                //return mouseEventHandledReturnValue;
            case EMIE_MOUSE_WHEEL:
                return mCurrentAppStateHandler->onMouseWheel(event.MouseInput);
                //return mouseEventHandledReturnValue;
            }
        }
        break;

    case EET_KEY_INPUT_EVENT:
        if (event.KeyInput.PressedDown)
            return mCurrentAppStateHandler->onKeyDown(event.KeyInput);
        else
            return mCurrentAppStateHandler->onKeyUp(event.KeyInput);
        //return true;

    case EET_LOG_TEXT_EVENT:
        // TODO - save log
        break;

    case EET_USER_EVENT:
        // Handle user event when necessary
        break;
    }


    /* 
     // TODO - app is closing down event - needs modifications to irrlicht (win32 WM_CLOSE)
     // ... setAlive(false);
     // TODO - focus lost/gain handling - needs modifications to irrlicht (win32 WM_ACTIVATE)
            case SDL_ACTIVEEVENT:
                if (mCurrentAppStateHandler != NULL &&
                    (event.active.state == SDL_APPINPUTFOCUS
                    // || event.active.state == SDL_APPACTIVE
                    ))
                {
                    //event.active.state = SDL_APPMOUSEFOCUS, SDL_APPINPUTFOCUS, SDL_APPACTIVE (iconify)
                    if (event.active.gain == 0)
                        mCurrentAppStateHandler->onFocusLost(event.active);
                    else if (event.active.gain == 1)
                        mCurrentAppStateHandler->onFocusGain(event.active);
                }
                break;
    */

    return false;
}


void App::setInstance(App *singleton)
{
    assert(mSingleton == NULL);
    mSingleton = singleton;
}


bool App::init(irr::IrrlichtDevice *irrDevice)
{
    setInstance(this);

    mIrrDevice = irrDevice;
    assert(mIrrDevice != NULL);
    if (mIrrDevice == NULL)
        return false;

    mIrrVideoDriver = mIrrDevice->getVideoDriver();
    assert(mIrrVideoDriver != NULL);
    if (mIrrVideoDriver == NULL)
        return false;
    mIrrSceneManager = mIrrDevice->getSceneManager();
    assert(mIrrSceneManager != NULL);
    if (mIrrSceneManager == NULL)
        return false;

    mGraphics = new GraphicsIrr(mIrrDevice);

    resetTime();
    srand((unsigned int)time(NULL));

    return true;
}


void App::run()
{
    /*
    // Warp mouse cursor position to middle of window/screen
    mMouseX = mPreviousMouseX = getScreenWidth() / 2;
    mMouseY = mPreviousMouseY = getScreenHeight() / 2;
    mIrrDevice->getCursorControl()->setPosition(mMouseX, mMouseY);
    */

    #ifdef _DEBUG
    int fpsCounterFrames = 0;
    #endif
    while (mIrrDevice->run() && isAlive())
    {
        irr::ITimer *irrTimer = mIrrDevice->getTimer();
        const UI32 frameUpdateStart = irrTimer->getRealTime();

        bool updated = false;
        I32 time = (I32)( /*frameUpdateStart*/ GetTickCount() - mStartTime);
        mTime = BLURMIX(mTime, time, 2); // smooth out the granularity just a bit

        updated |= update();

        if (updated)
        {
            if (mCurrentAppStateHandler != NULL)
                mCurrentAppStateHandler->render(mIrrDevice);
            render();

            mPreviousMouseX = mMouseX;
            mPreviousMouseY = mMouseY;
        }

        const UI32 frameUpdateEnd = irrTimer->getRealTime();
        if (frameUpdateStart == frameUpdateEnd)
            mIrrDevice->sleep(2);
        else
            mIrrDevice->sleep(1);

        #ifdef _DEBUG
        if (++fpsCounterFrames == 100)
        {
            core::stringw str = L"[Driver: ";
            str += mIrrDevice->getVideoDriver()->getName();
            str += L"] FPS: ";
            str += (s32)mIrrDevice->getVideoDriver()->getFPS();
            mIrrDevice->setWindowCaption(str.c_str());
            fpsCounterFrames = 0;
        }
        #endif // _DEBUG
    }
}


bool App::update()
{
    const double time = (double)mTime;

    if (time < mNextUpdateTime)
        return false;

    if (mCurrentAppStateHandler == NULL)
    {
        mNextUpdateTime = time;
        return false;
    }

    bool updated = false;
    int updateNum = 0;
    while (mNextUpdateTime < time && ++updateNum <= mMaxLogicUpdatesPerFrame)
    {
        const double prevTime = mNextUpdateTime;
        mNextUpdateTime += 1000.0 / (double)mCurrentAppStateHandler->getUpdateFPS();

        mCurrentAppStateHandler->update((I32)(mNextUpdateTime - prevTime));
    }

    return true;
}


void App::resetTime()
{
    mNextUpdateTime = 0;
    //assert(mIrrDevice && mIrrDevice->getTimer());
    mStartTime = GetTickCount(); //mIrrDevice->getTimer()->getRealTime();
    mTime = 0;
}


void App::setAppStateHandler(AppState *state)
{
    if (mCurrentAppStateHandler != NULL)
        mCurrentAppStateHandler->onDeactivate();

    mCurrentAppStateHandler = state;
    if (mCurrentAppStateHandler != NULL)
        mCurrentAppStateHandler->onActivate();

    resetTime();
}


wchar_t * App::getWindowCaption()
{
    return WINDOW_CAPTION_STR;
}
