/* Copyright 2009 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: MainState.cpp 103 2009-12-11 18:56:44Z tonic $
 * $Revision: 103 $
 */

#include <assert.h>
#include <algorithm>

#include "main.h"
#include "MainState.h"


enum SAMPLE_ID
{
    SAMPLE_START = 0,
    SAMPLE_COUNT
};

static char *sSampleFileNames[SAMPLE_COUNT] =
{
    "data/start.ogg"
};

#define AUDIO_CHANNELS 32

static FMUSIC_MODULE *sMusic;
static FSOUND_SAMPLE *sSamples[SAMPLE_COUNT];


MainState::MainState()
{
}


MainState::~MainState()
{
}


static bool sStringStartsWith(const c8 *str, const c8 *beginning)
{
    irr::core::stringc s(str), b(beginning);
    return s.subString(0, b.size()) == b;
}


bool MainState::init()
{
    Graphics *g = App::getInstance()->getGraphics();

    mTitleUIActive = true;
    mInGame = false;

    if (!FSOUND_Init(44100, AUDIO_CHANNELS, 0))
    {
        gErrorMessage("Audio error", "Unable to initialize sound! (%d)",
                      FSOUND_GetError());
    }
    for (u32 a = 0; a < SAMPLE_COUNT; ++a)
    {
        sSamples[a] = FSOUND_Sample_Load(FSOUND_FREE, sSampleFileNames[a], FSOUND_LOOP_OFF, 0, 0);
        if (!sSamples[a])
            gErrorMessage("Audio error", "Can't load %s", sSampleFileNames[a]);
    }
    sMusic = FMUSIC_LoadSong("data/n2000.s3m");
    if (!sMusic)
        gErrorMessage("Audio error", "Can't load music");
    FMUSIC_PlaySong(sMusic);

    mFont = new BMFont(g, "data/droid30.fnt");
    if (!mFont->init())
        return false;

    // Make sure these are in synch with IMAGE_ID
    mImages.push_back(g->loadImage("data/button.png"));
    mImages.push_back(g->loadImage("data/button-hot.png"));
    mImages.push_back(g->loadImage("data/button-active.png"));
    mImages.push_back(g->loadImage("data/button-hot-active.png"));

    mButtonStyle = new IMGUIImageButtonStyle(mFont, mImages[IMG_BUTTON], mImages[IMG_BUTTON_HOT], mImages[IMG_BUTTON_ACTIVE], mImages[IMG_BUTTON_HOT_ACTIVE]);

    mLabelText = "";

    mDevice = App::getInstance()->getIrrDevice();
    mVideo = App::getInstance()->getIrrVideoDriver();
    mSmgr = App::getInstance()->getIrrSceneManager();


    mCamera = mSmgr->addCameraSceneNodeMaya(0, -50, 10, 30);
    //mCamera = mSmgr->addCameraSceneNode();
    mCamera->setNearValue(0.1f);
    mCamera->setFarValue(1200);
    mCamera->setPosition(core::vector3df(0, 10, -10));
    mCamera->setTarget(core::vector3df(0, 10, 0));
    mCamera->setUpVector(core::vector3df(0, 1, 0));
    mSmgr->setActiveCamera(mCamera);

    video::ITexture *skyDomeTex = mVideo->getTexture("data/sky.png");
    scene::ISceneNode *skyDomeNode = mSmgr->addSkyDomeSceneNode(skyDomeTex, 16, 8, 1, 2);
    //skyDomeNode->getMaterial(0).setFlag(video::EMF_BILINEAR_FILTER, false);
    skyDomeNode->getMaterial(0).getTextureMatrix(0).setScale(core::vector3df(4, 2, 1));

    mSmgr->addLightSceneNode(0, core::vector3df(0, 50, 0));

    SMaterial planeMaterial;
    planeMaterial.Shininess = 0;
    planeMaterial.SpecularColor.set(0);
    planeMaterial.DiffuseColor.set(0xff708090);
    planeMaterial.AmbientColor.set(0xff708090);
    IMesh *planeMesh = mSmgr->addHillPlaneMesh("adf", dimension2df(1, 1), dimension2d<u32>(50, 50), &planeMaterial)->getMesh(0);
    ISceneNode *planeSN = mSmgr->addMeshSceneNode(planeMesh);
    planeSN->setAutomaticCulling(EAC_OFF);

    //mSmgr->loadScene("data/scene.irr");

    mLastMousePos.set(WIDTH / 2, HEIGHT / 2);

    mScore = 0;

    return mFont != NULL && mButtonStyle != NULL;
}


void MainState::deinit()
{
    UI32 a;
    delete mFont;
    for (a = 0; a < mImages.size(); ++a)
        delete mImages[a];
    mImages.clear();

    FMUSIC_StopAllSongs();
    FMUSIC_FreeSong(sMusic);
    for (a = 0; a < AUDIO_CHANNELS; ++a)
        FSOUND_StopSound(a);
    for (a = 0; a < SAMPLE_COUNT; ++a)
        FSOUND_Sample_Free(sSamples[a]);
    FSOUND_Close();
}


void MainState::onActivate(UI32 time)
{
    AppState::onActivate();
    mPaused = false;
}


bool MainState::onKeyUp(const irr::SEvent::SKeyInput & /*event*/)
{
    return false;
}

bool MainState::onKeyDown(const irr::SEvent::SKeyInput &event)
{
    if (event.Key == KEY_ESCAPE)
        App::getInstance()->setAlive(false);

    return false;
}

bool MainState::onMouseDown(const irr::SEvent::SMouseInput & /*event*/)
{
    if (mTitleUIActive || mEndingUIActive)
        mIMGUI.onMouseDown();

    return false;
}

bool MainState::onMouseUp(const irr::SEvent::SMouseInput & /*event*/)
{
    mIMGUI.onMouseUp();
    return false;
}

bool MainState::onMouseMotion(const irr::SEvent::SMouseInput &event)
{
    mLastMousePos.set(event.X, event.Y);
    mIMGUI.onMouseMotion(event.X, event.Y);
    return false;
}

bool MainState::onMouseWheel(const irr::SEvent::SMouseInput & /*event*/)
{
    return false;
}


bool MainState::update(UI32 time)
{
    bool result = AppState::update(time);
    if (!result)
        return false; // game is paused...

    if (mTitleUIActive)
    {
        return false;
    }

    /*
    if (mInGame && game_should_end)
    {
        mInGame = false;
        mEndingUIActive = true;
        //FSOUND_PlaySound(FSOUND_FREE, sSamples[SAMPLE_END]);
    }
    */

    // Following is executed only when game is not paused & game is running.

    // ...

    return true;
}


void MainState::render(irr::IrrlichtDevice *device)
{
    assert(device);
    irr::video::IVideoDriver *driver = device->getVideoDriver();
    assert(driver);
    irr::scene::ISceneManager *smgr = device->getSceneManager();

    driver->beginScene(true, true, video::SColor(0x00445566));

    smgr->drawAll();

    Graphics *g = App::getInstance()->getGraphics();
    g->beginRender();
    mIMGUI.begin(mStateTime);

    if (mTitleUIActive)
    {
        mFont->drawString(WIDTH / 2, 150, "game name", IF_ALIGN_HORIZ_CENTER, 1, Color(0xffffffff));
        mFont->drawString(WIDTH / 2, 180, "...", IF_ALIGN_HORIZ_CENTER, 0.7f, Color(0x80ffffff));

        mFont->drawString(WIDTH / 2, 250, "info", IF_ALIGN_HORIZ_CENTER, 0.7f, Color(0xaaffffff));
        mFont->drawString(WIDTH / 2, 280, "here", IF_ALIGN_HORIZ_CENTER, 0.7f, Color(0xaaffffff));

        if (mIMGUI.button(WIDTH / 2 - mButtonStyle->getWidth() / 2, 350, IMGUI_AI_COUNTER, mButtonStyle, 0, 0, "Play"))
        {
            mTitleUIActive = false;
            mInGame = true;
            mGameLogicTime = 0;
            FSOUND_PlaySound(FSOUND_FREE, sSamples[SAMPLE_START]);
        }
        if (mIMGUI.button(WIDTH / 2 - mButtonStyle->getWidth() / 2, 450, IMGUI_AI_COUNTER, mButtonStyle, 0, 0, "Quit"))
            App::getInstance()->setAlive(false);

        mFont->drawString(WIDTH / 2, HEIGHT - 5, "Copyright 2009 Jetro Lauha - www.jet.ro", IF_ALIGN_BOTTOM_HCENTER, 0.5f, Color(0x80ffffff));
    }
    else if (mInGame)
    {
        char buf[100];
        sprintf(buf, "%d", mScore);
        mFont->drawString(40, 40, "score", IF_ALIGN_TOP_LEFT, 0.7f, Color(0x88000000));
        mFont->drawString(40, 60, buf, IF_ALIGN_TOP_LEFT, 1, Color(0xff000000));
    }
    else if (mEndingUIActive)
    {
        char *title = "Game Over";
        char buf[100];
        sprintf(buf, "%d", mScore);
        if (mScore < 0)
            title = "Failboat!";
        mFont->drawString(WIDTH / 2, 200, title, IF_ALIGN_HORIZ_CENTER, 1, Color(0xff000000));
        mFont->drawString(WIDTH / 2, 300, "Final score:", IF_ALIGN_HORIZ_CENTER, 0.6f, Color(0x80000000));
        mFont->drawString(WIDTH / 2, 350, buf, IF_ALIGN_HORIZ_CENTER, 0.8f, Color(0xcc000000));
        if (mIMGUI.button(WIDTH / 2 - mButtonStyle->getWidth() / 2, 450, IMGUI_AI_COUNTER, mButtonStyle, 0, 0, "Quit"))
            App::getInstance()->setAlive(false);
    }

    mIMGUI.end();
    g->endRender();

    driver->endScene();
}
